@@ -1,15 +1,28 @@
+/*
+*  C Implementation: file_properties
+*
+* Description:
+*
+*
+* Author: Hong Jen Yee (PCMan) <pcman.tw (AT) gmail.com>, (C) 2006
+*
+* Copyright: See COPYING file that comes with this distribution
+*
+*/
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
+#include "private.h"
+
 #include <gtk/gtk.h>
 #include "glib-mem.h"
 
 #include "file-properties.h"
 #include "file-properties-ui.h"
-#include "glade-support.h"
 
-#include "xdgmime.h"
+#include "mime-type/mime-type.h"
 
 #include <sys/types.h>
 #include <pwd.h>
@@ -136,11 +149,11 @@ gboolean on_update_labels( FilePropertiesDialogData* data )
 
     gdk_threads_enter();
 
-    file_size_to_string( buf2, data->total_size );
+    vfs_file_size_to_string( buf2, data->total_size );
     sprintf( buf, "%s  (%llu Bytes)", buf2, ( guint64 ) data->total_size );
     gtk_label_set_text( data->total_size_label, buf );
 
-    file_size_to_string( buf2, data->size_on_disk );
+    vfs_file_size_to_string( buf2, data->size_on_disk );
     sprintf( buf, "%s  (%llu Bytes)", buf2, ( guint64 ) data->size_on_disk );
     gtk_label_set_text( data->size_on_disk_label, buf );
 
@@ -285,13 +298,13 @@ GtkWidget* file_properties_dlg_new( GtkWindow* parent,
 
     const char* multiple_files = _( "Multiple files are selected" );
     const char* calculating;
-    GtkWidget* name = lookup_widget( dlg, "file_name" );
-    GtkWidget* location = lookup_widget( dlg, "location" );
-    GtkWidget* mime_type = lookup_widget( dlg, "mime_type" );
-    GtkWidget* open_with = lookup_widget( dlg, "open_with" );
+    GtkWidget* name = g_object_get_data( G_OBJECT(dlg), "file_name" );
+    GtkWidget* location = g_object_get_data( G_OBJECT(dlg), "location" );
+    GtkWidget* mime_type = g_object_get_data( G_OBJECT(dlg), "mime_type" );
+    GtkWidget* open_with = g_object_get_data( G_OBJECT(dlg), "open_with" );
 
-    GtkWidget* mtime = lookup_widget( dlg, "mtime" );
-    GtkWidget* atime = lookup_widget( dlg, "atime" );
+    GtkWidget* mtime = g_object_get_data( G_OBJECT(dlg), "mtime" );
+    GtkWidget* atime = g_object_get_data( G_OBJECT(dlg), "atime" );
 
     char buf[ 64 ];
     char buf2[ 32 ];
@@ -320,14 +333,14 @@ GtkWidget* file_properties_dlg_new( GtkWindow* parent,
     gtk_label_set_text( GTK_LABEL( location ), disp_path );
     g_free( disp_path );
 
-    data->total_size_label = GTK_LABEL( lookup_widget( dlg, "total_size" ) );
-    data->size_on_disk_label = GTK_LABEL( lookup_widget( dlg, "size_on_disk" ) );
-    data->owner = GTK_ENTRY( lookup_widget( dlg, "owner" ) );
-    data->group = GTK_ENTRY( lookup_widget( dlg, "group" ) );
+    data->total_size_label = GTK_LABEL( g_object_get_data( G_OBJECT(dlg), "total_size" ) );
+    data->size_on_disk_label = GTK_LABEL( g_object_get_data( G_OBJECT(dlg), "size_on_disk" ) );
+    data->owner = GTK_ENTRY( g_object_get_data( G_OBJECT(dlg), "owner" ) );
+    data->group = GTK_ENTRY( g_object_get_data( G_OBJECT(dlg), "group" ) );
 
     for ( i = 0; i < N_CHMOD_ACTIONS; ++i )
     {
-        data->chmod_btns[ i ] = GTK_TOGGLE_BUTTON( lookup_widget( dlg, chmod_names[ i ] ) );
+        data->chmod_btns[ i ] = GTK_TOGGLE_BUTTON( g_object_get_data( G_OBJECT(dlg), chmod_names[ i ] ) );
     }
 
     for ( l = sel_files; l && l->next; l = l->next )
@@ -375,7 +388,7 @@ GtkWidget* file_properties_dlg_new( GtkWindow* parent,
     {
         /* if open with shouldn't show, destroy it. */
         gtk_widget_destroy( open_with );
-        gtk_widget_destroy( lookup_widget( dlg, "open_with_label" ) );
+        gtk_widget_destroy( g_object_get_data( G_OBJECT(dlg), "open_with_label" ) );
     }
     else /* Add available actions to the option menu */
     {
@@ -478,7 +491,7 @@ GtkWidget* file_properties_dlg_new( GtkWindow* parent,
                      ( guint64 ) vfs_file_info_get_size( file ) );
             gtk_label_set_text( data->total_size_label, buf );
 
-            file_size_to_string( buf2,
+            vfs_file_size_to_string( buf2,
                                  vfs_file_info_get_blocks( file ) * 512 );
             sprintf( buf, "%s  (%llu Bytes)", buf2,
                      ( guint64 ) vfs_file_info_get_blocks( file ) * 512 );
@@ -627,7 +640,7 @@ on_filePropertiesDlg_response ( GtkDialog *dialog,
             GtkWidget* open_with;
 
             /* Set default action for mimetype */
-            if( open_with = lookup_widget( GTK_WIDGET(dialog), "open_with" ) )
+            if( open_with = g_object_get_data( G_OBJECT( GTK_WIDGET(dialog) ), "open_with" ) )
             {
                 GtkTreeModel* model = gtk_combo_box_get_model( GTK_COMBO_BOX(open_with) );
                 GtkTreeIter it;
@@ -654,18 +667,18 @@ on_filePropertiesDlg_response ( GtkDialog *dialog,
                 uid = uid_from_name( owner_name );
                 if ( uid == -1 )
                 {
-                    ptk_show_error( GTK_WINDOW( dialog ), _( "Invalid User" ) );
+                    ptk_show_error( GTK_WINDOW( dialog ), _("Error"), _( "Invalid User" ) );
                     return ;
                 }
             }
             group_name = gtk_entry_get_text( data->group );
-            if ( group_name && *group_name && 
+            if ( group_name && *group_name &&
                  (!data->group_name || strcmp( group_name, data->group_name )) )
             {
                 gid = gid_from_name( group_name );
                 if ( gid == -1 )
                 {
-                    ptk_show_error( GTK_WINDOW( dialog ), _( "Invalid Group" ) );
+                    ptk_show_error( GTK_WINDOW( dialog ), _("Error"), _( "Invalid Group" ) );
                     return ;
                 }
             }
