@@ -10,6 +10,10 @@
 *
 */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "ptk-file-menu.h"
 #include <glib.h>
 #include "glib-mem.h"
@@ -17,6 +21,8 @@
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 
+#include <unistd.h> /* for access */
+
 #include "vfs-app-desktop.h"
 #include "ptk-utils.h"
 #include "ptk-file-misc.h"
@@ -170,7 +176,7 @@ GtkWidget* ptk_file_menu_new( const char* file_path,
 
     GtkWidget *open;
     char* open_title;
-    GtkWidget *open_with, *open_with_menu, *open_with_another;
+    GtkWidget *open_with, *cut, *copy, *del, *rename, *open_with_menu, *open_with_another;
     GtkWidget *seperator;
     GtkWidget *create_new;
     GtkWidget *extract;
@@ -189,13 +195,13 @@ GtkWidget* ptk_file_menu_new( const char* file_path,
     GtkWidget* app_img;
 
     PtkFileMenu* data;
+    int no_write_access = 0;
 
     data = g_slice_new0( PtkFileMenu );
     data->cwd = g_strdup( cwd );
     data->browser = browser;
     data->file_path = g_strdup( file_path );
-    data->info = info;
-    vfs_file_info_ref( info );
+    data->info = vfs_file_info_ref( info );
     data->sel_files = sel_files;
 
     data->accel_group = gtk_accel_group_new ();
@@ -220,17 +226,46 @@ GtkWidget* ptk_file_menu_new( const char* file_path,
     }
 
     basic_popup_menu[ 0 ].ret = &open_with;
+    basic_popup_menu[ 2 ].ret = &cut;
+    basic_popup_menu[ 3 ].ret = &copy;
     basic_popup_menu[ 4 ].ret = &paste;
+    basic_popup_menu[ 5 ].ret = &del;
+    basic_popup_menu[ 6 ].ret = &rename;
     basic_popup_menu[ 9 ].ret = &extract;
     basic_popup_menu[ 10 ].ret = &create_new;
     ptk_menu_add_items_from_data( popup, basic_popup_menu, data, data->accel_group );
     gtk_widget_show_all( GTK_WIDGET( popup ) );
 
+#if defined(HAVE_EUIDACCESS)
+    no_write_access = euidaccess( file_path, W_OK );
+#elif defined(HAVE_EACCESS)
+    no_write_access = eaccess( file_path, W_OK );
+#endif
+
+    if( no_write_access )
+    {
+        /* have no write access to current file */
+        gtk_widget_set_sensitive( cut, FALSE );
+        gtk_widget_set_sensitive( copy, FALSE );
+        gtk_widget_set_sensitive( paste, FALSE );
+        gtk_widget_set_sensitive( del, FALSE );
+        gtk_widget_set_sensitive( rename, FALSE );
+        gtk_widget_set_sensitive( create_new, FALSE );
+    }
+
     if ( ! is_dir )
     {
         gtk_widget_destroy( paste );
         gtk_widget_destroy( create_new );
     }
+    else if( 0 == strcmp( file_path, cwd ) ) /* menu for cwd */
+    {
+        /* delete some confusing menu item if this menu is for current folder */
+        gtk_widget_destroy( cut );
+        gtk_widget_destroy( copy );
+        gtk_widget_destroy( del );
+        gtk_widget_destroy( rename );
+    }
 
     /*  Add all of the apps  */
     open_with_menu = gtk_menu_new ();
@@ -248,8 +283,10 @@ GtkWidget* ptk_file_menu_new( const char* file_path,
         {
             len1 = apps ? g_strv_length( apps ) : 0;
             len2 = g_strv_length( txt_apps );
+            tmp = apps;
             apps = vfs_mime_type_join_actions( apps, len1, txt_apps, len2 );
             g_strfreev( txt_apps );
+            g_strfreev( tmp );
         }
         vfs_mime_type_unref( txt_type );
     }
@@ -604,13 +641,16 @@ create_new_file( PtkFileMenu* data, gboolean create_dir )
         char* cwd;
         GtkWidget* parent;
         parent = gtk_widget_get_toplevel( GTK_WIDGET(data->browser) );
-        if( data->file_path && 
+        if( data->file_path &&
             g_file_test( data->file_path, G_FILE_TEST_IS_DIR ) )
             cwd = data->file_path;
         else
             cwd = data->cwd;
-        ptk_create_new_file( GTK_WINDOW( parent ),
-                             cwd, create_dir );
+        if( G_LIKELY(data->browser) )
+            ptk_file_browser_create_new_file( data->browser, create_dir );
+        else /* Is this possible? */
+            ptk_create_new_file( GTK_WINDOW( parent ),
+                                 cwd, create_dir, NULL );
     }
 }
 
