@@ -11,7 +11,6 @@
 */
 
 #include "vfs-file-info.h"
-#include "xdgmime.h"
 #include <glib.h>
 #include "glib-mem.h"
 #include <glib/gi18n.h>
@@ -20,9 +19,7 @@
 #include <string.h>
 
 #include "vfs-app-desktop.h"
-#include "md5.h"    /* for thumbnails */
-
-#define _MAX( x, y )     (x > y ? x : y)
+#include "vfs-thumbnail-loader.h"
 
 static int big_thumb_size = 48, small_thumb_size = 20;
 static gboolean utf8_file_name = FALSE;
@@ -87,15 +84,15 @@ static void vfs_file_info_clear( VFSFileInfo* fi )
     fi->flags = VFS_FILE_INFO_NONE;
 }
 
-void vfs_file_info_ref( VFSFileInfo* fi )
+VFSFileInfo* vfs_file_info_ref( VFSFileInfo* fi )
 {
-    ++fi->n_ref;
+    g_atomic_int_inc( &fi->n_ref );
+    return fi;
 }
 
 void vfs_file_info_unref( VFSFileInfo* fi )
 {
-    --fi->n_ref;
-    if ( fi->n_ref <= 0 )
+    if ( g_atomic_int_dec_and_test( &fi->n_ref) )
     {
         vfs_file_info_clear( fi );
         g_slice_free( VFSFileInfo, fi );
@@ -179,7 +176,7 @@ const char* vfs_file_info_get_disp_size( VFSFileInfo* fi )
     if ( G_UNLIKELY( !fi->disp_size ) )
     {
         char buf[ 64 ];
-        file_size_to_string( buf, fi->size );
+        vfs_file_size_to_string( buf, fi->size );
         fi->disp_size = g_strdup( buf );
     }
     return fi->disp_size;
@@ -247,7 +244,7 @@ GdkPixbuf* vfs_file_info_get_big_icon( VFSFileInfo* fi )
         else
             w = h = 0;
 
-        if ( _MAX( w, h ) != icon_size )
+        if ( ABS( MAX( w, h ) - icon_size ) > 2 )
         {
             char * icon_name = NULL;
             if ( fi->big_thumbnail )
@@ -374,7 +371,7 @@ const char* vfs_file_info_get_disp_perm( VFSFileInfo* fi )
     return fi->disp_perm;
 }
 
-void file_size_to_string( char* buf, guint64 size )
+void vfs_file_size_to_string( char* buf, guint64 size )
 {
     char * unit;
     /* guint point; */
@@ -473,13 +470,13 @@ gboolean vfs_file_info_is_unknown_type( VFSFileInfo* fi )
 /* full path of the file is required by this function */
 gboolean vfs_file_info_is_executable( VFSFileInfo* fi, const char* file_path )
 {
-    return xdg_mime_is_executable_file( file_path, fi->mime_type->type );
+    return mime_type_is_executable_file( file_path, fi->mime_type->type );
 }
 
 /* full path of the file is required by this function */
 gboolean vfs_file_info_is_text( VFSFileInfo* fi, const char* file_path )
 {
-    return xdg_mime_is_text_file( file_path, fi->mime_type->type );
+    return mime_type_is_text_file( file_path, fi->mime_type->type );
 }
 
 /*
@@ -531,35 +528,8 @@ mode_t vfs_file_info_get_mode( VFSFileInfo* fi )
     return fi->mode;
 }
 
-static void unload_thumnails_if_needed( VFSFileInfo* fi )
-{
-    int w, h;
-    if ( fi->big_thumbnail && fi->flags == VFS_FILE_INFO_NONE )
-    {
-        w = gdk_pixbuf_get_width( fi->big_thumbnail );
-        h = gdk_pixbuf_get_height( fi->big_thumbnail );
-        if ( _MAX( w, h ) != big_thumb_size )
-        {
-            gdk_pixbuf_unref( fi->big_thumbnail );
-            fi->big_thumbnail = NULL;
-        }
-    }
-    if ( fi->small_thumbnail && fi->flags == VFS_FILE_INFO_NONE )
-    {
-        w = gdk_pixbuf_get_width( fi->small_thumbnail );
-        h = gdk_pixbuf_get_height( fi->small_thumbnail );
-        if ( MAX( w, h ) != small_thumb_size )
-        {
-            gdk_pixbuf_unref( fi->small_thumbnail );
-            fi->small_thumbnail = NULL;
-        }
-    }
-}
-
 gboolean vfs_file_info_is_thumbnail_loaded( VFSFileInfo* fi, gboolean big )
 {
-    /* FIXME: I cannot find a better place to unload thumbnails */
-    unload_thumnails_if_needed( fi );
     if ( big )
         return ( fi->big_thumbnail != NULL );
     return ( fi->small_thumbnail != NULL );
@@ -569,22 +539,11 @@ gboolean vfs_file_info_load_thumbnail( VFSFileInfo* fi,
                                        const char* full_path,
                                        gboolean big )
 {
-    char * uri;
-    md5_byte_t md5[ 16 ];
-    char* thumbnail_file;
-    char file_name[ 36 ];
-    char mtime_str[ 32 ];
-    const char* thumb_mtime;
-    int i, w, h, size;
-    struct stat statbuf;
-    GdkPixbuf* thumbnail, *result = NULL;
-
-    /* FIXME: I cannot find a better place to unload thumbnails */
-    unload_thumnails_if_needed( fi );
+    GdkPixbuf* thumbnail;
 
     if ( big )
     {
-        if ( fi->big_thumbnail && big )
+        if ( fi->big_thumbnail )
             return TRUE;
     }
     else
@@ -592,87 +551,23 @@ gboolean vfs_file_info_load_thumbnail( VFSFileInfo* fi,
         if ( fi->small_thumbnail )
             return TRUE;
     }
-
-    if ( !gdk_pixbuf_get_file_info( full_path, &w, &h ) )
-        return FALSE;   /* image format cannot be recognized */
-
-    /* If the image itself is very small, we should load it directly */
-    if ( w <= 128 && h <= 128 )
+    thumbnail = vfs_thumbnail_load_for_file( full_path,
+                                                    big ? big_thumb_size : small_thumb_size , fi->mtime );
+    if( G_LIKELY( thumbnail ) )
     {
-        size = big ? big_thumb_size : small_thumb_size;
-        thumbnail = gdk_pixbuf_new_from_file_at_scale( full_path, size, size,
-                                                       TRUE, NULL );
         if ( big )
             fi->big_thumbnail = thumbnail;
         else
             fi->small_thumbnail = thumbnail;
-        return ( thumbnail != NULL );
     }
-
-    uri = g_filename_to_uri( full_path, NULL, NULL );
-
-    md5_state_t md5_state;
-    md5_init( &md5_state );
-    md5_append( &md5_state, ( md5_byte_t * ) uri, strlen( uri ) );
-    md5_finish( &md5_state, md5 );
-
-    for ( i = 0; i < 16; ++i )
+    else /* fallback to mime_type icon */
     {
-        sprintf( ( file_name + i * 2 ), "%02x", md5[ i ] );
-    }
-    strcpy( ( file_name + i * 2 ), ".png" );
-
-    thumbnail_file = g_build_filename( g_get_home_dir(),
-                                       ".thumbnails/normal",
-                                       file_name, NULL );
-
-    /* load existing thumbnail */
-    thumbnail = gdk_pixbuf_new_from_file( thumbnail_file, NULL );
-    if ( !thumbnail ||
-            !( thumb_mtime = gdk_pixbuf_get_option( thumbnail, "tEXt::Thumb::MTime" ) ) ||
-            atol( thumb_mtime ) != fi->mtime )
-    {
-        /* create new thumbnail */
-        thumbnail = gdk_pixbuf_new_from_file_at_size( full_path, 128, 128, NULL );
-        if ( thumbnail )
-        {
-            sprintf( mtime_str, "%lu", fi->mtime );
-            gdk_pixbuf_save( thumbnail, thumbnail_file, "png", NULL,
-                             "tEXt::Thumb::URI", uri, "tEXt::Thumb::MTime", mtime_str, NULL );
-        }
-    }
-
-    if ( thumbnail )
-    {
-        w = gdk_pixbuf_get_width( thumbnail );
-        h = gdk_pixbuf_get_height( thumbnail );
-        size = big ? big_thumb_size : small_thumb_size;
-        if ( w > h )
-        {
-            h = h * size / w;
-            w = size;
-        }
-        else if ( h > w )
-        {
-            w = w * size / h;
-            h = size;
-        }
-        else
-        {
-            w = h = size;
-        }
-        result = gdk_pixbuf_scale_simple(
-                     thumbnail,
-                     w, h, GDK_INTERP_BILINEAR );
-        gdk_pixbuf_unref( thumbnail );
         if ( big )
-            fi->big_thumbnail = result;
+            fi->big_thumbnail = vfs_file_info_get_big_icon( fi );
         else
-            fi->small_thumbnail = result;
+            fi->small_thumbnail = vfs_file_info_get_small_icon( fi );
     }
-    g_free( uri );
-    g_free( thumbnail_file );
-    return ( result != NULL );
+    return ( thumbnail != NULL );
 }
 
 void vfs_file_info_set_thumbnail_size( int big, int small )
